﻿namespace HeuristicLab.Problems.ProgramSynthesis.Push.Expressions {
  using System.Collections.Generic;

  using HeuristicLab.Problems.ProgramSynthesis.Push.Attributes;

  using Interpreter;
  using Stack;

  public abstract class RotateExpression<T> : StatelessExpression {
    public bool Eval(IPushStack<T> stack) {
      if (stack.Count < 3) return false;

      stack.Swap(3);
      return true;
    }
  }


  [PushExpression(StackTypes.Integer, "INTEGER.ROT")]
  public class IntegerRotateExpression : RotateExpression<long> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.IntegerStack);
    }
  }

  [PushExpression(StackTypes.Float, "FLOAT.ROT")]
  public class FloatRotateExpression : RotateExpression<double> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.FloatStack);
    }
  }

  [PushExpression(StackTypes.Boolean, "BOOLEAN.ROT")]
  public class BooleanRotateExpression : RotateExpression<bool> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.BooleanStack);
    }
  }

  [PushExpression(StackTypes.Name, "NAME.ROT")]
  public class NameRotateExpression : RotateExpression<string> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.NameStack);
    }
  }

  [PushExpression(StackTypes.Exec, "EXEC.ROT")]
  public class ExecRotateExpression : RotateExpression<Expression> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.ExecStack);
    }
  }

  [PushExpression(StackTypes.Code, "CODE.ROT")]
  public class CodeRotateExpression : RotateExpression<Expression> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.CodeStack);
    }
  }

  [PushExpression(StackTypes.Char, "CHAR.ROT")]
  public class CharRotateExpression : RotateExpression<char> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.CharStack);
    }
  }

  [PushExpression(StackTypes.String, "STRING.ROT")]
  public class StringRotateExpression : RotateExpression<string> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.StringStack);
    }
  }

  [PushExpression(StackTypes.IntegerVector, "INTEGER[].ROT")]
  public class IntegerVectorRotateExpression : RotateExpression<List<long>> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.IntegerVectorStack);
    }
  }

  [PushExpression(StackTypes.FloatVector, "FLOAT[].ROT")]
  public class FloatVectorRotateExpression : RotateExpression<List<double>> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.FloatVectorStack);
    }
  }

  [PushExpression(StackTypes.BooleanVector, "BOOLEAN[].ROT")]
  public class BooleanVectorRotateExpression : RotateExpression<List<bool>> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.BooleanVectorStack);
    }
  }

  [PushExpression(StackTypes.StringVector, "STRING[].ROT")]
  public class StringVectorRotateExpression : RotateExpression<List<string>> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.StringVectorStack);
    }
  }
}